/************************************************************************//**
 *
 * @file    rfc_reg.h
 *
 * @brief   Definition of the RFC Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef RFC_REG_H__
#define RFC_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// RFC Ch.N Clock Control Register
typedef union RFCNCLK_REG_TAG {
    unsigned short  reg;
    struct RFCNCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of the RFC.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the RFC.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the RFC operating clock.
         * +---------------+-----------------------------------------+
         * |               |                CLKSRC[1:0]              |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | CLKDIV[1:0]   +---------+----------+---------+----------+
         * |               | IOSCCLK |   OSC1   | OSC3CLK |  EXOSC   |
         * +---------------+---------+----------+---------+----------+
         * |      0x3      | 1/8     |   1/1    | 1/8     |  1/1     |
         * +---------------+---------+          +---------+          +
         * |      0x2      | 1/4     |          | 1/4     |          |
         * +---------------+---------+          +---------+          +
         * |      0x1      | 1/2     |          | 1/2     |          |
         * +---------------+---------+          +---------+          +
         * |      0x0      | 1/1     |          | 1/1     |          |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the RFC operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the RFC.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the RFC operating clock.
         * +---------------+-----------------------------------------+
         * |               |                CLKSRC[1:0]              |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | CLKDIV[1:0]   +---------+----------+---------+----------+
         * |               | IOSCCLK |   OSC1   | OSC3CLK |  EXOSC   |
         * +---------------+---------+----------+---------+----------+
         * |      0x3      | 1/8     |   1/1    | 1/8     |  1/1     |
         * +---------------+---------+          +---------+          +
         * |      0x2      | 1/4     |          | 1/4     |          |
         * +---------------+---------+          +---------+          +
         * |      0x1      | 1/2     |          | 1/2     |          |
         * +---------------+---------+          +---------+          +
         * |      0x0      | 1/1     |          | 1/1     |          |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * This bit sets whether the RFC operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the RFC operating clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNCLK_REG;

/// RFC Ch.N Control Register
typedef union RFCNCTL_REG_TAG {
    unsigned short  reg;
    struct RFCNCTL_REG_BIT_TAG {
        /**
         * This bit enables the RFC operations.
         * .MODEN
         *  This bit enables the RFC operations.
         *    1 (R/W): Enable RFC operations (The operating clock is supplied.)
         *    0 (R/W): Disable RFC operations (The operating clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * These bits configure the oscillation mode.
         * .SMODE[1:0]
         *  These bits configure the oscillation mode.
         * +------------+------------------------------------------------------+
         * | SMODE[1:0] | Interrupt/reset generating condition                 |
         * +------------+------------------------------------------------------+
         * |  0x3,0x2   | reserved.                                            |
         * +------------+------------------------------------------------------+
         * |      0x1   |AC oscillation mode for resistive sensor measurements |
         * +------------+------------------------------------------------------+
         * |      0x0   |DC oscillation mode for resistive sensor measurements |
         * +------------+------------------------------------------------------+
         */
        unsigned short      SMODE       : 2;
        /**
         * This bit enables external clock input mode (event counter mode).
         * .EVTEN
         *  This bit enables external clock input mode (event counter mode).
         *    1 (R/W): External clock input mode
         *    0 (R/W): Normal mode
         */
        unsigned short      EVTEN       : 1;
        /**
         * This bit disables the automatic CR oscillation stop function to enable continuous oscillation function.
         * .CONEN
         *  This bit disables the automatic CR oscillation stop function to enable continuous oscillation function.
         *    1 (R/W): Enable continuous oscillation
         *    0 (R/W): Disable continuous oscillation
         */
        unsigned short      CONEN       : 1;
        /**
         * This bit sets the RFCLKOn pin to output the divided-by-two oscillation clock.
         * .RFCLKMD
         *  This bit sets the RFCLKOn pin to output the divided-by-two oscillation clock.
         *    1 (R/W): Divided-by-two clock output
         *    0 (R/W): Oscillation clock output
         */
        unsigned short      RFCLKMD     : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNCTL_REG;

/// RFC Ch.N Oscillation Trigger Register
typedef union RFCNTRG_REG_TAG {
    unsigned short  reg;
    struct RFCNTRG_REG_BIT_TAG {
        /**
         * This bit controls CR oscillation for the reference resistor. This bit also indicates the CR oscillation status.
         * .SREF
         *  This bit controls CR oscillation for the reference resistor. This bit also indicates the CR oscillation status.
         *    1 (W): Start oscillation
         *    0 (W): Stop oscillation
         *    1 (R): Being oscillated
         *    0 (R): Stopped
         */
        unsigned short      SREF        : 1;
        /**
         * This bit controls CR oscillation for sensor A. This bit also indicates the CR oscillation status.
         * .SSENA
         *  This bit controls CR oscillation for sensor A. This bit also indicates the CR oscillation status.
         *    1 (W): Start oscillation
         *    0 (W): Stop oscillation
         *    1 (R): Being oscillated
         *    0 (R): Stopped
         */
        unsigned short      SSENA       : 1;
        /**
         * This bit controls CR oscillation for sensor B. This bit also indicates the CR oscillation status.
         * .SSENB
         *  This bit controls CR oscillation for sensor B. This bit also indicates the CR oscillation status.
         *    1 (W): Start oscillation
         *    0 (W): Stop oscillation
         *    1 (R): Being oscillated
         *    0 (R): Stopped
         */
        unsigned short      SSENB       : 1;
        unsigned short      reserved_1  :13;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNTRG_REG;

/// RFC Ch.N Measurement Counter Low Register
typedef union RFCNMCL_REG_TAG {
    unsigned short  reg;
    struct RFCNMCL_REG_BIT_TAG {
        /// Measurement counter data can be read and written through these bits.
        unsigned short      MC          :16;
    } bit;
    unsigned char   byte_reg[2];
} RFCNMCL_REG;

/// RFC Ch.N Measurement Counter High Register
typedef union RFCNMCH_REG_TAG {
    unsigned short  reg;
    struct RFCNMCH_REG_BIT_TAG {
        /// Measurement counter data can be read and written through these bits.
        unsigned short      MC          : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNMCH_REG;

/// RFC Ch.N Measurement Counter Register
typedef union RFCNMC_DWORD_REG_TAG {
    unsigned long   reg;
} RFCNMC_DWORD_REG;

/// RFC Ch.N Time Base Counter Low Register
typedef union RFCNTCL_REG_TAG {
    unsigned short  reg;
    struct RFCNTCL_REG_BIT_TAG {
        /// Time base counter data can be read and written through these bits.
        unsigned short      TC          :16;
    } bit;
    unsigned char   byte_reg[2];
} RFCNTCL_REG;

/// RFC Ch.N Time Base Counter High Register
typedef union RFCNTCH_REG_TAG {
    unsigned short  reg;
    struct RFCNTCH_REG_BIT_TAG {
        /// Time base counter data can be read and written through these bits.
        unsigned short      TC          : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNTCH_REG;

/// RFC Ch.N Time Base Counter Register
typedef union RFCNTC_DWORD_REG_TAG {
    unsigned long   reg;
} RFCNTC_DWORD_REG;

/// RFC Ch.N Interrupt Flag Register
typedef union RFCNINTF_REG_TAG {
    unsigned short  reg;
    struct RFCNINTF_REG_BIT_TAG {
        /**
         * This bit indicate Reference oscillation completion interrupt cause occurrence status.
         * .EREFIF
         *  This bit indicate Reference oscillation completion interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      EREFIF      : 1;
        /**
         * This bit indicate Sensor B oscillation completion interrupt cause occurrence status.
         * .ESENAIF
         *  This bit indicate Sensor B oscillation completion interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      ESENAIF     : 1;
        /**
         * This bit indicate Sensor B oscillation completion interrupt cause occurrence status.
         * .ESENBIF
         *  This bit indicate Sensor B oscillation completion interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      ESENBIF     : 1;
        /**
         * This bit indicate Measurement counter overflow error interrupt cause occurrence status.
         * .OVMCIF
         *  This bit indicate Measurement counter overflow error interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      OVMCIF      : 1;
        /**
         * This bit indicate Time base counter overflow error interrupt cause occurrence status.
         * .OVTCIF
         *  This bit indicate Time base counter overflow error interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      OVTCIF      : 1;
        unsigned short      reserved_1  :11;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNINTF_REG;

/// RFC Ch.N Interrupt Enable Register
typedef union RFCNINTE_REG_TAG {
    unsigned short  reg;
    struct RFCNINTE_REG_BIT_TAG {
        /**
         * This bit enable Reference oscillation completion interrupt.
         * .EREFIE
         *  This bit enable Reference oscillation completion interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      EREFIE      : 1;
        /**
         * This bit enable Sensor A oscillation completion interrupt.
         * .ESENAIE
         *  This bit enable Sensor A oscillation completion interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      ESENAIE     : 1;
        /**
         * This bit enable Sensor B oscillation completion interrupt.
         * .ESENBIE
         *  This bit enable Sensor B oscillation completion interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      ESENBIE     : 1;
        /**
         * This bit enable Measurement counter overflow error interrupt.
         * .OVMCIE
         *  This bit enable Measurement counter overflow error interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      OVMCIE      : 1;
        /**
         * This bit enable Time base counter overflow error interrupt.
         * .OVTCIE
         *  This bit enable Time base counter overflow error interrupt.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      OVTCIE      : 1;
        unsigned short      reserved_1  :11;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} RFCNINTE_REG;

#ifdef __cplusplus
}
#endif

#endif // RFC_REG_H__
