/************************************************************************//**
 *
 * @file    svd3_reg.h
 *
 * @brief   Definition of the Supply Voltage Detector(SVD3) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef SVD3_REG_H__
#define SVD3_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// SVD3 Clock Control Register
typedef union SVDCLK_REG_TAG {
    unsigned short  reg;
    struct SVDCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of SVD3.
         * .CLKSRC[1:0]
         *  These bits select the clock source of SVD3.
         * .CLKDIV[2:0]
         *  These bits select the division ratio of the SVD3 operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * |             |  0x0   |   0x1  |  0x2   |  0x3   |
         * | CLKDIV[2:0] +--------+--------+--------+--------+
         * |             |        |        |        |        |
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * |             |        |        |        |        |
         * +-------------+--------+--------+--------+--------+
         * |   0x7,0x6   |Reserved|1/1     |Reserved|1/1     |
         * +-------------+--------+        +--------+        +
         * |     0x5     |1/512   |        |1/512   |        |
         * +-------------+--------+        +--------+        +
         * |     0x4     |1/256   |        |1/256   |        |
         * +-------------+--------+        +--------+        +
         * |     0x3     |1/128   |        |1/128   |        |
         * +-------------+--------+        +--------+        +
         * |     0x2     |1/64    |        |1/64    |        |
         * +-------------+--------+        +--------+        +
         * |     0x1     |1/32    |        |1/32    |        |
         * +-------------+--------+        +--------+        +
         * |     0x0     |1/16    |        |1/16    |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the SVD3 operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of SVD3.
         * .CLKDIV[2:0]
         *  These bits select the division ratio of the SVD3 operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * |             |  0x0   |   0x1  |  0x2   |  0x3   |
         * | CLKDIV[2:0] +--------+--------+--------+--------+
         * |             |        |        |        |        |
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * |             |        |        |        |        |
         * +-------------+--------+--------+--------+--------+
         * |   0x7,0x6   |Reserved|1/1     |Reserved|1/1     |
         * +-------------+--------+        +--------+        +
         * |     0x5     |1/512   |        |1/512   |        |
         * +-------------+--------+        +--------+        +
         * |     0x4     |1/256   |        |1/256   |        |
         * +-------------+--------+        +--------+        +
         * |     0x3     |1/128   |        |1/128   |        |
         * +-------------+--------+        +--------+        +
         * |     0x2     |1/64    |        |1/64    |        |
         * +-------------+--------+        +--------+        +
         * |     0x1     |1/32    |        |1/32    |        |
         * +-------------+--------+        +--------+        +
         * |     0x0     |1/16    |        |1/16    |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKDIV      : 3;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit sets whether the SVD3 operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the SVD3 operating clock is supplied in DEBUG mode or not.
         *    1 (R/WP): Clock supplied in DEBUG mode
         *    0 (R/WP): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SVDCLK_REG;

/// SVD3 Control Register
typedef union SVDCTL_REG_TAG {
    unsigned short  reg;
    struct SVDCTL_REG_BIT_TAG {
        /**
         * This bit enables/disables for the SVD3 circuit to operate.
         * .MODEN
         *  This bit enables/disables for the SVD3 circuit to operate.
         *    1 (R/WP): Enable (Start detection operations)
         *    0 (R/WP): Disable (Stop detection operations)
         */
        unsigned short      MODEN       : 1;
        /**
         * These bits select intermittent operation mode and its detection cycle.
         * .SVDMD[1:0]
         *  These bits select intermittent operation mode and its detection cycle.
         * +------------+--------------------------------------------+
         * | SVDMD[1:0] | Operation mode (detection cycle)           |
         * +------------+--------------------------------------------+
         * |    0x3     | Intermittent operation mode (CLK_SVD3/512) |
         * +------------+--------------------------------------------+
         * |    0x2     | Intermittent operation mode (CLK_SVD3/256) |
         * +------------+--------------------------------------------+
         * |    0x1     | Intermittent operation mode (CLK_SVD3/128) |
         * +------------+--------------------------------------------+
         * |    0x0     | Continuous operation mode                  |
         * +------------+--------------------------------------------+
         */
        unsigned short      SVDMD       : 2;
        /**
         * This bit selects the external voltage to be detected when the SVDCTL.VDSEL bit = 1.
         * .EXSEL
         *  This bit selects the external voltage to be detected when the SVDCTL.VDSEL bit = 1.
         *    1 (R/WP): EXSVD1
         *    0 (R/WP): EXSVD0
         */
        unsigned short      EXSEL       : 1;
        /**
         * These bits enable/disable the reset issuance function when a low power supply voltage is detected.
         * .SVDRE[3:0]
         *  These bits enable/disable the reset issuance function when a low power supply voltage is detected.
         *    0xa (R/WP): Enable (Issue reset)
         *    Other than 0xa (R/WP): Disable (Generate interrupt)
         */
        unsigned short      SVDRE       : 4;
        /**
         * These bits select an SVD3 detection voltage VSVD  for detecting low voltage.
         * .SVDC[4:0]
         *  These bits select an SVD detection voltage VSVD/EXSVD detection voltage VSVD_EXT for detecting
         *  low voltage.
         * +-----------+--------------------------------------+
         * | SVDC[4:0] | SVD detection voltage VSVD/          |
         * |           | EXSVD detection voltage VSVD_EXT [V] |
         * +-----------+--------------------------------------+
         * |   0x1f    |                 High.                |
         * +-----------+                                      +
         * |   0x1e    |                                      |
         * +-----------+                                      +
         * |   0x1d    |                                      |
         * +-----------+                                      +
         * |    :      |                                      |
         * +-----------+                                      +
         * |   0x02    |                                      |
         * +-----------+                                      +
         * |   0x01    |                                      |
         * +-----------+                                      +
         * |   0x00    |                 Low.                 |
         * +-----------+--------------------------------------+
         */
        unsigned short      SVDC        : 5;
        /**
         * These bits set the condition to generate an interrupt/reset in intermittent operation mode.
         *.SVDSC[1:0]
         * These bits set the condition to generate an interrupt/reset (number of successive low voltage detections)
         * in intermittent operation mode (SVDCTL.SVDMD[1:0] bits = 0x1 to 0x3).
         *+------------+----------------------------------------------------------------+
         *| SVDSC[1:0] | Interrupt/reset generating condition                           |
         *+------------+----------------------------------------------------------------+
         *|    0x3     | Low power supply voltage is successively detected eight times. |
         *+------------+----------------------------------------------------------------+
         *|    0x2     | Low power supply voltage is successively detected four times.  |
         *+------------+----------------------------------------------------------------+
         *|    0x1     | Low power supply voltage is successively detected twice.       |
         *+------------+----------------------------------------------------------------+
         *|    0x0     | Low power supply voltage is successively detected once.        |
         *+------------+----------------------------------------------------------------+
         */
        unsigned short      SVDSC       : 2;
        /**
         * This bit selects the power supply voltage to be detected by SVD3.
         * .VDSEL
         *  This bit selects the power supply voltage to be detected by SVD3.
         *    1 (R/WP): Voltage applied to the EXSVD0/1 pin
         *    0 (R/WP): VDD
         */
        unsigned short      VDSEL       : 1;
    } bit;
    unsigned char   byte_reg[2];
} SVDCTL_REG;

/// SVD3 Status and Interrupt Flag Register
typedef union SVDINTF_REG_TAG {
    unsigned short  reg;
    struct SVDINTF_REG_BIT_TAG {
        /**
         * This bit indicates the low power supply voltage detection interrupt cause occurrence status.
         * .SVDIF
         *  This bit indicates the low power supply voltage detection interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      SVDIF       : 1;
        unsigned short      reserved_1  : 7;    ///< Reserved.
        /**
         * The power supply voltage detection results can be read out from this bit.
         * .SVDDT
         *  The power supply voltage detection results can be read out from this bit.
         *    1 (R): Power supply voltage (VDD or EXSVD0/1) < SVD detection voltage VSVD
         *                                                    or EXSVD detection voltage VSVD_EXT
         *    0 (R): Power supply voltage (VDD or EXSVD0/1) >= SVD detection voltage VSVD
         *                                                    or EXSVD detection voltage VSVD_EXT
         */
        unsigned short      SVDDT       : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SVDINTF_REG;

/// SVD3 Interrupt Enable Register
typedef union SVDINTE_REG_TAG {
    unsigned short  reg;
    struct SVDINTE_REG_BIT_TAG {
        /**
         * This bit enables low power supply voltage detection interrupts.
         * .SVDIE
         *  This bit enables low power supply voltage detection interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      SVDIE       : 1;
        unsigned short      reserved_1  :15;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SVDINTE_REG;

#ifdef __cplusplus
}
#endif

#endif // SVD3_REG_H__
