/************************************************************************//**
 *
 * @file    uart3_reg.h
 *
 * @brief   Definition of the UART (UART3) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef UART3_REG_H__
#define UART3_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// UART3 Ch.N Clock Control Register
typedef union UANCLK_REG_TAG {
    unsigned short  reg;
    struct UANCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of the UART3.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the UART3.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the UART3 operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * |             |  0x0   |   0x1  |  0x2   |  0x3   |
         * | CLKDIV[1:0] +--------+--------+--------+--------+
         * |             |        |        |        |        |
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * |             |        |        |        |        |
         * +-------------+--------+--------+--------+--------+
         * |     0x3     |1/8     |1/1     |1/8     |1/1     |
         * +-------------+--------+        +--------+        +
         * |     0x2     |1/4     |        |1/4     |        |
         * +-------------+--------+        +--------+        +
         * |     0x1     |1/2     |        |1/2     |        |
         * +-------------+--------+        +--------+        +
         * |     0x0     |1/1     |        |1/1     |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the UART3 operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the UART3.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the UART3 operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * |             |  0x0   |   0x1  |  0x2   |  0x3   |
         * | CLKDIV[1:0] +--------+--------+--------+--------+
         * |             |        |        |        |        |
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * |             |        |        |        |        |
         * +-------------+--------+--------+--------+--------+
         * |     0x3     |  1/8   |  1/1   |  1/8   |  1/1   |
         * +-------------+--------+        +--------+        +
         * |     0x2     |  1/4   |        |  1/4   |        |
         * +-------------+--------+        +--------+        +
         * |     0x1     |  1/2   |        |  1/2   |        |
         * +-------------+--------+        +--------+        +
         * |     0x0     |  1/1   |        |  1/1   |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * This bit sets whether the UART3 operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the UART3 operating clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANCLK_REG;

/// UART3 Ch.N Mode Register
typedef union UANMOD_REG_TAG {
    unsigned short  reg;
    struct UANMOD_REG_BIT_TAG {
        /**
         * This bit sets the stop bit length.
         * .STPB
         *  This bit sets the stop bit length.
         *    1 (R/W): 2 bits
         *    0 (R/W): 1 bit
         */
        unsigned short      STPB        : 1;
        /**
         * This bit selects either odd parity or even parity when using the parity function.
         * .PRMD
         *  This bit selects either odd parity or even parity when using the parity function.
         *    1 (R/W): Odd parity
         *    0 (R/W): Even parity
         */
        unsigned short      PRMD        : 1;
        /**
         * This bit sets the data length.
         * .PREN
         *  This bit enables the parity function.
         *    1 (R/W): Enable parity function
         *    0 (R/W): Disable parity function
         */
        unsigned short      PREN        : 1;
        /**
         * This bit sets the data length.
         * .CHLN
         *  This bit sets the data length.
         *    1 (R/W): 8 bits
         *    0 (R/W): 7 bits
         */
        unsigned short      CHLN        : 1;
        /**
         * This bit enables the IrDA interface function.
         * .IRMD
         *  This bit enables the IrDA interface function.
         *    1 (R/W): Enable IrDA interface function
         *    0 (R/W): Disable IrDA interface function
         */
        unsigned short      IRMD        : 1;
        /**
         * This bit sets the USOUT0 pin output mode.
         * .OUTMD
         *  This bit sets the USOUTn pin output mode.
         *    1 (R/W): Open-drain output
         *    0 (R/W): Push-pull output
         */
        unsigned short      OUTMD       : 1;
        /**
         * This bit enables pull-up of the USIN0 pin.
         * .PUEN
         *  This bit enables pull-up of the USINn pin.
         *    1 (R/W): Enable pull-up
         *    0 (R/W): Disable pull-up
         */
        unsigned short      PUEN        : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit enables the USOUT0 output inverting function.
         * .INVTX
         *  This bit enables the USOUTn output inverting function.
         *    1 (R/W): Enable output inverting function
         *    0 (R/W): Disable output inverting function
         */
        unsigned short      INVTX       : 1;
        /**
         * This bit enables the USIN0 input inverting function.
         * .INVRX
         *  This bit enables the USINn input inverting function.
         *    1 (R/W): Enable input inverting function
         *    0 (R/W): Disable input inverting function
         */
        unsigned short      INVRX       : 1;
        /**
         * This bit sets the UART3 operating clock division ratio for generating the transfer (sampling) clock using the baud rate generator.
         * .BRDIV
         *  This bit sets the UART3 operating clock division ratio for generating the transfer (sampling) clock
         *  using the baud rate generator.
         *    1 (R/W): 1/4
         *    0 (R/W): 1/16
         */
        unsigned short      BRDIV       : 1;
        /**
         * This bit enables the carrier modulation function.
         * .CAREN
         *  This bit enables the carrier modulation function.
         *    1 (R/W): Enable carrier modulation function
         *    0 (R/W): Disable carrier modulation function
         */
        unsigned short      CAREN       : 1;
        /**
         * This bit selects the carrier modulation period.
         * .PECAR
         *  This bit selects the carrier modulation period.
         *    1 (R/W): Carrier modulation during H data period
         *    0 (R/W): Carrier modulation during L data period
         */
        unsigned short      PECAR       : 1;
        unsigned short      reserved_2  : 3;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANMOD_REG;

/// UART3 Ch.N Baud-Rate Register
typedef union UANBR_REG_TAG {
    unsigned short  reg;
    struct UANBR_REG_BIT_TAG {
        /**
         * These bits set the UART3 transfer rate.
         * .BRT[7:0]
         *  These bits set the UART3 transfer rate. For more information, refer to <Baud Rate Generator>.
         */
        unsigned short      BRT         : 8;
        /**
         * These bits set the UART3 transfer rate.
         * .FMD[3:0]
         *  These bits set the UART3 transfer rate.
         */
        unsigned short      FMD         : 4;
        unsigned short      reserved_1  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANBR_REG;

/// UART3 Ch.N Control Register
typedef union UANCTL_REG_TAG {
    unsigned short  reg;
    struct UANCTL_REG_BIT_TAG {
        /**
         * This bit enables the UART3 operations.
         * .MODEN
         *  This bit enables the UART3 operations.
         *    1 (R/W): Enable UART3 operations (The operating clock is supplied.)
         *    0 (R/W): Disable UART3 operations (The operating clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        /**
         * This bit issues software reset to the UART3.
         * .SFTRST
         *  This bit issues software reset to the UART3.
         *    1 (W): Issue software reset
         *    0 (W): Ineffective
         *    1 (R): Software reset is executing.
         *    0 (R): Software reset has finished. (During normal operation)
         */
        unsigned short      SFTRST      : 1;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANCTL_REG;

/// UART3 Ch.N Transmit Data Register
typedef union UANTXD_REG_TAG {
    unsigned short  reg;
    struct UANTXD_REG_BIT_TAG {
        /**
         * Data can be written to the transmit data buffer through these bits.
         * .TXD[7:0]
         *  Data can be written to the transmit data buffer through these bits. Make sure the UAnINTF.TBEIF bit
         *  is set to 1 before writing data.
         */
        unsigned short      TXD         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANTXD_REG;

/// UART3 Ch.N Receive Data Register
typedef union UANRXD_REG_TAG {
    unsigned short  reg;
    struct UANRXD_REG_BIT_TAG {
        /**
         * The receive data buffer can be read through these bits.
         * .RXD[7:0]
         *  The receive data buffer can be read through these bits. The receive data buffer consists of a 2-byte
         *  FIFO, and older received data is read first.
         */
        unsigned short      RXD         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANRXD_REG;

/// UART3 Ch.N Status and Interrupt Flag Register
typedef union UANINTF_REG_TAG {
    unsigned short  reg;
    struct UANINTF_REG_BIT_TAG {
        /**
         * These bits indicate Transmit buffer empty interrupt cause occurrence status.
         * .TBEIF
         *  These bits indicate the UART3 interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .TBEIF bit: Transmit buffer empty interrupt
         */
        unsigned short      TBEIF       : 1;
        /**
         * These bits indicate Receive buffer one byte full interrupt cause occurrence status.
         * .RB1FIF
         *  Receive buffer one byte full interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      RB1FIF      : 1;
        /**
         * These bits indicate Receive buffer two bytes full interrupt cause occurrence status.
         * .RB2FIF
         *  Receive buffer two bytes full interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      RB2FIF      : 1;
        /**
         * These bits indicate Overrun error interrupt cause occurrence status.
         * .OEIF
         *  Overrun error interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      OEIF        : 1;
        /**
         * These bits indicate Parity error interrupt cause occurrence status.
         * .PEIF
         *  Parity error interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      PEIF        : 1;
        /**
         * These bits indicate Framing error interrupt cause occurrence status.
         * .FEIF
         *  Framing error interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      FEIF        : 1;
        /**
         * These bits indicate End-of-transmission interrupt cause occurrence status.
         * .TENDIF
         *  End-of-transmission interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      TENDIF      : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit indicates the sending status.
         * .TBSY
         *  This bit indicates the sending status. (See Figure 11.5.2.1.)
         *    1 (R): During sending
         *    0 (R): Idle
         */
        unsigned short      TBSY        : 1;
        /**
         * This bit indicates the receiving status.
         * .RBSY
         *  This bit indicates the receiving status.
         *    1 (R): During receiving
         *    0 (R): Idle
         */
        unsigned short      RBSY        : 1;
        unsigned short      reserved_2  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANINTF_REG;

/// UART3 Ch.N Interrupt Enable Register
typedef union UANINTE_REG_TAG {
    unsigned short  reg;
    struct UANINTE_REG_BIT_TAG {
        /**
         * These bits enable Transmit buffer empty interrupt.
         * .TBEIE
         *  These bits enable UART3 interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      TBEIE       : 1;
        /**
         * These bits enable Receive buffer one byte full interrupt.
         * .RB1FIE
         *  Receive buffer one byte full interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      RB1FIE      : 1;
        /**
         * These bits enable Receive buffer two bytes full interrupt.
         * .RB2FIE
         *  Receive buffer two bytes full interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      RB2FIE      : 1;
        /**
         * These bits enable Overrun error interrupt.
         * .OEIE
         *  Overrun error interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      OEIE        : 1;
        /**
         * These bits enable Parity error interrupt.
         * .PEIE
         *  Parity error interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      PEIE        : 1;
        /**
         * These bits enable Framing error interrupt.
         * .FEIE
         *  Framing error interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      FEIE        : 1;
        /**
         * These bits enable End-of-transmission interrupt.
         * .TENDIE
         *  End-of-transmission interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      TENDIE      : 1;
        unsigned short      reserved_1  : 9;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANINTE_REG;

/// UART3 Ch.N Carrier Waveform Register
typedef union UANCAWF_REG_TAG {
    unsigned short  reg;
    struct UANCAWF_REG_BIT_TAG {
        /**
         * These bits set the carrier modulation output frequency.
         * .CRPER[7:0]
         *   These bits set the carrier modulation output frequency.
         */
        unsigned short      CRPER       : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} UANCAWF_REG;

#ifdef __cplusplus
}
#endif

#endif // UART3_REG_H__
