/************************************************************************//**
 *
 * @file    c17_svd3.c
 *
 * @brief   This is Supply voltage detector program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_svd3.h"
#include "c17_init_config.h"


#if defined(C17_PE_SVD)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector04_handler, c17intSvd);		// SVD interrupt
#endif /* CRT0_LINKED */


/* --- global variable --- */

// Interrupt flag.
volatile unsigned int c17svdIntLowVoltage = 0;	///< Low voltage interrupt.


/************************************************************************//**
 * @brief  Initialize SVD.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initSvd(void)
{
	SVDCLK_REG	svdclk_set;
	SVDCTL_REG	svdctl_set;

	/// Stop SVD.
	c17stopSvd();

	/// Disable write-protect.
	c17protectMiscOff();

	/// Set clock source for SVD.
	/* SVDCLK:SVD3 Clock Control Register */
	{
	    /**
	     * @todo Change clock source & Clock division ratio selection
	     */
		svdclk_set.bit.DBRUN = 1;					/// - Clock supplied in DEBUG mode.
		svdclk_set.bit.CLKDIV = C17_SVD_CLKDIV_16;	/// - Set division ratio for clock source(1/16).
		svdclk_set.bit.CLKSRC = C17_SVD_IOSC;		/// - Set clock source(IOSC).
		SVDCLK = svdclk_set.reg;
	}

	/* SVDCTL:SVD3 Control Register */
	{
	    /**
	     * @todo Change SVD mode SVDMD,EXSEL,SVDRE,SVDSC,VDSEL
	     */
		svdctl_set.bit.VDSEL = 0;					/// VDD
		svdctl_set.bit.SVDSC = 0;					/// Low power supply voltage is successively detected once.
		svdctl_set.bit.SVDC = C17_SVDC_2700MV;		//  Comparison voltage for detecting low voltage(2.7V).
		svdctl_set.bit.SVDRE = 0;					/// Select reset(0xA)/interrupt(others) mode.
		svdctl_set.bit.EXSEL = 0;					/// EXSVD0
		svdctl_set.bit.SVDMD = 0;					/// Intermittent operation mode(0x1-0x3) / Continuous operation mode(0x0).
		svdctl_set.bit.MODEN = 0;					/// SVD disable
		SVDCTL = svdctl_set.reg;
	}
	/// Enable write-protect.
	c17protectMiscOn();

	/// Initialize the interrupt.
	ILVSVD	= INTERRUPT_LEVEL_SVD;
}


/************************************************************************//**
 * @brief   Start SVD.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startSvd(void)
{
	SVDINTE_REG	svdinte_set;
	SVDCTL_REG	svdctl_set;
	SVDINTF_REG	svdintf_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/// Clear interrupt flag.
	/* SVDINTF: SVD3 Status and Interrupt Flag Register */
	{
		svdintf_set.reg = 0;
		svdintf_set.bit.SVDIF = 1;
		SVDINTF	= svdintf_set.reg;
	}

	/// Enable interrupt.
	/* SVDINTE:SVD3 Interrupt Enable Register */
	{
		svdinte_set.reg = 0;
		svdinte_set.bit.SVDIE = 1;
		SVDINTE = svdinte_set.reg;
	}

	/// Start SVD.
	/* SVDCTL:SVD3 Control Register */
	{
		svdctl_set.reg = SVDCTL;
	    svdctl_set.bit.MODEN = 1;
		SVDCTL = svdctl_set.reg;

	}
	/// Enable write-protect.
	c17protectMiscOn();
}


/************************************************************************//**
 * @brief   Stop SVD.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopSvd(void)
{
	SVDINTE_REG	svdinte_set;
	SVDCTL_REG	svdctl_set;
	SVDINTF_REG	svdintf_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/// Stop SVD.
	/* SVDCTL:SVD3 Control Register */
	{
		svdctl_set.reg = SVDCTL;
	    svdctl_set.bit.MODEN = 0;
		SVDCTL = svdctl_set.reg;
	}

	/// Disable interrupt.
	/* SVDINTE:SVD3 Interrupt Enable Register */
	{
		svdinte_set.reg = 0;
		svdinte_set.bit.SVDIE = 0;
		SVDINTE = svdinte_set.reg;
	}

	/// Clear interrupt flag.
	/* SVDINTF: SVD3 Status and Interrupt Flag Register */
	{
		svdintf_set.reg = 0;
		svdintf_set.bit.SVDIF = 1;
		SVDINTF	= svdintf_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();
}


/************************************************************************//**
 * @brief   Set Comparison voltage for detecting low voltage.
 *
 * @param   svdVolt       Comparison voltage for detecting low voltage.
 *
 * @retval  none
 ***************************************************************************/
void c17setSvdComparisonVoltage(unsigned short svdVolt)
{
	SVDCTL_REG	svdctl_set;

	/* SVDCTL:SVD3 Control Register */
	{
		/// Disable write-protect.
		c17protectMiscOff();

		svdctl_set.reg = SVDCTL;
		svdctl_set.bit.SVDC = svdVolt;
		SVDCTL = svdctl_set.reg;

		/// Enable write-protect.
		c17protectMiscOn();
	}
}


/************************************************************************//**
 * @brief   Get power supply voltage detection results.
 *
 * @param   none
 *
 * @retval  C17_SVD_SUPPLY_LOW    Power supply voltage(VDD or EXSVD) >=  comparison voltage.
 * @retval  C17_SVD_SUPPLY_HIGH   Power supply voltage(VDD or EXSVD) <  comparison voltage.
 ***************************************************************************/
unsigned short c17getSvdVoltageDetection(void)
{
	SVDINTF_REG	svdintf_seg;

	/// SVD3 Status and Interrupt Flag Register
	svdintf_seg.reg = SVDINTF;

	// Check Power supply voltage(VDD or EXSVD) to comparison voltage.
	return  svdintf_seg.bit.SVDDT;
}


/************************************************************************//**
 * @brief   SVD interrupt.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intSvd(void)
{
	SVDINTF_REG	svdintf_seg;

	/// SVD3 Status and Interrupt Flag Register
	svdintf_seg.reg = SVDINTF;
	/// Clear interrupt flag.
	SVDINTF = svdintf_seg.reg;

	/// Check interrupt cause occurrence status.
	if (svdintf_seg.bit.SVDIF == 1) {

	   /**
		* @todo add interrupt function of SVD
		*/
		/// Set Low voltage interrupt flag.
		c17svdIntLowVoltage = 1;
	}
}
#endif

