/************************************************************************//**
 *
 * @file	main.c
 *
 * @brief	This is SVD3 sample program for S1C17M20/M21/M22/M23/M24/M25 demonstration.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *          - 01 Sep. 2016    1.00
 *				- First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ***************************************************************************/
/************************************************************************//**
 * @main page	S1C17M20/M21/M22/M23/M24/M25 Supply Voltage Detector (SVD3)
 *
 * @section		intro Introduction
 * Sample code for detect the supply Voltage for ten seconds.
 * Acquire the frequency that became a low power.
 *
 * @subsection	step1	Step 1: Initialize T16 Ch.0
 * @subsection	step2	Step 2: Initialize and start SVD.
 * @subsection	step3	Step 3: Set Comparison voltage for detecting low voltage(2.7V).
 * @subsection	step4	Step 4: Start T16 Ch.0 (1-second interrupt).
 * @subsection	step5	Step 5: Get supply voltage detection by T16 Ch.0 1sec Interrupt(10 seconds).
 * @subsection	step6	Step 6: Stop T16 Ch.0.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17m20_m21_m22_m23_m24_m25_sample" project.
 *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_t16.h"
#include "c17_svd3.h"


#ifdef C17_PE_SVD
/* --- global variable --- */


extern volatile unsigned int c17t16Ch0IntCount;
extern volatile unsigned int c17svdIntLowVoltage;

/* --- function declaration --- */
int main_svd3(void);



/************************************************************************//**
 * main function.
 *
 * @brief	1,SVD register is initialized.
 *
 *          2,Get supply voltage detection by T16 1sec Interrupt(10 counts).
 *
 *
 *
 * @return 	EXIT_SUCCESS	Success.
 * @return 	EXIT_FAILURE	Failed.
 ***************************************************************************/
int main_svd3(void)
{
	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();
	c17stopClgOSC3();

	/// start OSC3(internal oscillator)
	c17startClgIntOSC3(C17_CLG_OSC3_12MHZ);
	c17setOsc3OpeInSleep(C17_CLG_CONTINUE);


	/* <<< Supply Voltage Detector demonstration start >>> */

	/// Clear Interruption T16 counter.
	c17t16Ch0IntCount = 0;

	/// Clear Interruption SVD counter.
	c17svdIntLowVoltage = 0;

	/**
	 * Clock Source=OSC3(12MHz)
	 * Clock Division = 1/512
	 * Ft = 1sec
	 * TR = 1[sec] * f((12MHz/512) -1 = 23437
	 */
	/// Initialize  T16 CH.0 timer.
	c17setT16(0, C17_T16_CLKSRC_OSC3, C17_T16_CLKDIV_512, C17_T16_MODE_REPEAT);
	/// Initialize the interrupt.
	ILVT16_0 = INTERRUPT_LEVEL_T16_0;				/// - Set interrupt level.

	///  T16 CH.0 Set Reload Value
	c17setCountT16(0, 23437);

	/// T16 CH.0 Interrupt Enable
	c17enableIntT16(0);

	/// Start T16 CH.0 timer.
	c17startT16(0);

	/// Initialize SVD.
	c17initSvd();

	/// Set Comparison voltage for detecting low voltage(2.7V).
	c17setSvdComparisonVoltage(C17_SVDC_2700MV);

	/// Start SVD.
	c17startSvd();

	/// Wait 10 Counts.
	while (c17t16Ch0IntCount < 10) {
		asm("halt");
		asm("nop");

		/// - Check SVD low voltage interrupt.
		if (c17svdIntLowVoltage == 1) {

			c17svdIntLowVoltage = 0;
			/// - Get power supply voltage detection results.
			if(c17getSvdVoltageDetection() == C17_SVD_SUPPLY_LOW)
			{
				/// SVD detecting low voltage(<2.7V).
				asm("nop");
			}
		}
	}
	/// Stop SVD.
	c17stopSvd();

	/// Stop T16 CH.0 timer.
	c17stopT16(0);

	/// Disable  T16 CH.0 timer.
	c17disableT16(0);



	/* <<< Supply Voltage Detector demonstration finish >>> */

	return EXIT_SUCCESS;
}


#endif
