/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is UART3 sample program for S1C17M20/M21/M22/M23/M24/M25 demonstration.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17M20/M21/M22/M23/M24/M25 UART3
 *
 * @section     intro Introduction
 * Sample code transmit data using UART3.
 * The code receive data using UART3.
 *
 * @subsection  --- MASTER ---
 * @subsection  step1   Step 1: Start UART3.
 *                      Initialize and Start UART3.
 * @subsection  step2   Step 2: Transmit "A" to "Z".
 * @subsection  step3   Step 3: Receive data.
 * @subsection  step4   Step 4: Compare receive data and the transmission data.
 *                      If it is the same data, OK is returned.
 *
 * @subsection  --- SLAVE ---
 * @subsection  step1   Step 1: Start UAR3T.
 *                      Initialize and Start UART3.
 * @subsection  step2   Step 2: Receive data.
 * @subsection  step3   Step 3: Transmit "A" to "Z".
 * @subsection  step4   Step 4: Compare receive data and the transmission data.
 *                      If it is the same data, OK is returned.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17m20_m21_m22_m23_m24_m25_sample" project.
 *
 *******************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_uart3.h"

#if defined(C17_PE_UART_MASTER) || defined(C17_PE_UART_SLAVE)

/* --- definition --- */

/* --- global variable --- */

/// Transmit data buffer.
static char sndData[28] =
{
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
	'\0','\0'
};
/// Receive data buffer.
static char rcvData[28];


/* --- function declaration --- */
int main_uart3(void);

#ifdef C17_PE_UART_MASTER
static int runMaster(void);
#else
static int runSlave(void);
#endif

/************************************************************************//**
 * Main function.
 *
 * @brief   1, Initialize and start UART3.
 *
 *          2, The processing of master or slave begins.
 *
 *          3, stop UART3
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int main_uart3(void)
{
	int i;
	int result;

	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();
	c17stopClgOSC3();

	/// Clear receive data buffer.
	for (i = 0; i < sizeof(rcvData); i++) {
		rcvData[i] = 0;
	}

	/* <<< UART3 demonstration start >>> */

	/// Initialize UART3.
	result = c17initUart();
	if(result == EXIT_FAILURE)
	{
		/// baud rate setting is overflow.
		asm("nop");
	}

	/// UART3 Start.
	c17startUart();

#ifdef C17_PE_UART_MASTER
	// UART3 Master.
	result = runMaster();
#else
	// UART3 Slave.
	result = runSlave();
#endif

	/// Stop UART3.
	c17stopUart();

	/* <<< UART3 demonstration finish >>> */


	return result;
}

#ifdef C17_PE_UART_MASTER
/************************************************************************//**
 * runMaster function.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int runMaster(void)
{
	int status;

	/* Transmit use interrupt */
	c17enableIntUart(C17_UART_RCV_1BYTE);

	status = c17sendUartData(sndData, sizeof(sndData));
	if (status != EXIT_SUCCESS) {
		return status;
	}
	while(c17sendUartDataWait() != EXIT_SUCCESS)
	{

	}

	/* Receive use interrupt */
	c17rcvUartaData(rcvData, sizeof(rcvData), C17_UART_RCV_1BYTE);
	do
	{
		status = c17rcvUartDataWait();
		switch(status)
		{
		   /**
			* @todo add error process.
			*/
			case C17_UART_ERROR_OVERFLOW:
				///< OverFlow Error
				break;
			case C17_UART_ERROR_PARITY:
				///< Parity Error
				break;
			case C17_UART_ERROR_FRAMING:
				///< Framing Error
				break;
		}
	}while(status != EXIT_SUCCESS);

	/* Check data */
	if (strcmp(sndData, rcvData) != 0) {
		return EXIT_FAILURE;
	}

	/* Transmit not use interrupt */
	c17disableIntUart();

	c17sendUartDataPoling(sndData, sizeof(sndData));

	/* Receive not use interrupt */
	c17rcvUartDataPolingByte(rcvData, sizeof(rcvData));

	/* Check data */
	if (strcmp(sndData, rcvData) != 0) {
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}

#else

/************************************************************************//**
 * runSlave function.
 *
 * @retval  TRUE    Success.
 * @retval  FALSE   Failed.
 ***************************************************************************/
int runSlave(void)
{
	int status;


	/* Receive not use interrupt */
	c17disableIntUart();
	c17rcvUartDataPolingByte(rcvData, sizeof(rcvData));

	/* Transmit not use interrupt */
	c17sendUartDataPoling(sndData, sizeof(sndData));

	/* Check data */
	if (strcmp(sndData, rcvData) != 0) {
		return EXIT_FAILURE;
	}

	/* Receive use interrupt */
	c17enableIntUart(C17_UART_RCV_1BYTE);
	c17rcvUartaData(rcvData, sizeof(rcvData), C17_UART_RCV_1BYTE);
	do
	{
		status = c17rcvUartDataWait();
		switch(status)
		{
		   /**
			* @todo add error process.
			*/
			case C17_UART_ERROR_OVERFLOW:
				///< OverFlow Error
				break;
			case C17_UART_ERROR_PARITY:
				///< Parity Error
				break;
			case C17_UART_ERROR_FRAMING:
				///< Framing Error
				break;
		}
	}while(status != EXIT_SUCCESS);

	/* Transmit use interrupt */
	status = c17sendUartData(sndData, sizeof(sndData));
	if (status != EXIT_SUCCESS) {
		return status;
	}
	while(c17sendUartDataWait() != EXIT_SUCCESS)
	{

	}

	/* Check data */
	if (strcmp(sndData, rcvData) != 0) {
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
#endif

#endif

